"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@checkup/core");
class OutdatedDependenciesTaskResult extends core_1.BaseTaskResult {
    constructor() {
        super(...arguments);
        this.actions = [];
    }
    process(data) {
        this.data = data;
        let actionsEvaluator = new core_1.ActionsEvaluator();
        actionsEvaluator.add({
            name: 'reduce-outdated-major-dependencies',
            summary: 'Update outdated major versions',
            details: `${this.versionTypes.get('major').length} major versions outdated`,
            defaultThreshold: 0.05,
            items: [],
            input: this.versionTypes.get('major').length / this.data.dependencies.length,
        });
        actionsEvaluator.add({
            name: 'reduce-outdated-minor-dependencies',
            summary: 'Update outdated minor versions',
            details: `${this.versionTypes.get('minor').length} minor versions outdated`,
            defaultThreshold: 0.05,
            items: [],
            input: this.versionTypes.get('minor').length / this.data.dependencies.length,
        });
        actionsEvaluator.add({
            name: 'reduce-outdated-dependencies',
            summary: 'Update outdated versions',
            details: `${core_1.toPercent(this.outdatedDependencies.length / this.data.dependencies.length)} of versions outdated`,
            defaultThreshold: 0.2,
            items: [],
            input: this.outdatedDependencies.length / this.data.dependencies.length,
        });
        this.actions = actionsEvaluator.evaluate(this.config);
    }
    toConsole() {
        core_1.ui.section(this.meta.friendlyTaskName, () => {
            core_1.ui.sectionedBar([
                { title: 'major', count: this.versionTypes.get('major').length, color: 'red' },
                { title: 'minor', count: this.versionTypes.get('minor').length, color: 'orange' },
                { title: 'patch', count: this.versionTypes.get('patch').length, color: 'yellow' },
            ], this.data.dependencies.length);
        });
    }
    toJson() {
        return {
            meta: this.meta,
            result: {
                dependencies: this.outdatedDependencies,
            },
        };
    }
    get outdatedDependencies() {
        // if a dependency is up to date, the `bump` field will be null
        return this.data.dependencies.filter((dependency) => dependency.bump !== null);
    }
    get versionTypes() {
        var _a;
        let versionTypes = new Map([
            ['major', []],
            ['minor', []],
            ['patch', []],
        ]);
        for (let dependency of this.outdatedDependencies) {
            try {
                // for catching when dependency version is 'exotic', which means yarn cannot detect for you whether the package has become outdated
                (_a = versionTypes.get(dependency.bump)) === null || _a === void 0 ? void 0 : _a.push(dependency);
            }
            catch (_b) {
                // do nothing
            }
        }
        return versionTypes;
    }
}
exports.default = OutdatedDependenciesTaskResult;
//# sourceMappingURL=outdated-dependencies-task-result.js.map